From 7deb372f97d14c8ed3a5a83a881f5bc5ed6a7009 Mon Sep 17 00:00:00 2001
From: Sergey Volkov <s.volkov@netris.ru>
Date: Tue, 5 Jun 2018 14:45:06 +0300
Subject: [PATCH] fix ActionBar CalledFromWrongThreadException

---
 .../appcelerator/titanium/proxy/ActivityProxy.java | 24 ++++++++++++++++++----
 1 file changed, 20 insertions(+), 4 deletions(-)

diff --git a/android/titanium/src/java/org/appcelerator/titanium/proxy/ActivityProxy.java b/android/titanium/src/java/org/appcelerator/titanium/proxy/ActivityProxy.java
index 8754b06..33b0a51 100644
--- a/android/titanium/src/java/org/appcelerator/titanium/proxy/ActivityProxy.java
+++ b/android/titanium/src/java/org/appcelerator/titanium/proxy/ActivityProxy.java
@@ -11,7 +11,9 @@ import org.appcelerator.kroll.KrollDict;
 import org.appcelerator.kroll.KrollFunction;
 import org.appcelerator.kroll.KrollProxy;
 import org.appcelerator.kroll.annotations.Kroll;
+import org.appcelerator.kroll.common.AsyncResult;
 import org.appcelerator.kroll.common.Log;
+import org.appcelerator.kroll.common.TiMessenger;
 import org.appcelerator.titanium.TiApplication;
 import org.appcelerator.titanium.TiBaseActivity;
 import org.appcelerator.titanium.TiC;
@@ -48,6 +50,7 @@ public class ActivityProxy extends KrollProxy implements TiActivityResultHandler
 	private static final int MSG_FIRST_ID = KrollProxy.MSG_LAST_ID + 1;
 	private static final int MSG_INVALIDATE_OPTIONS_MENU = MSG_FIRST_ID + 100;
 	private static final int MSG_OPEN_OPTIONS_MENU = MSG_FIRST_ID + 101;
+	private static final int MSG_GET_ACTIONBAR = MSG_FIRST_ID + 102;
 
 	protected Activity wrappedActivity;
 	protected IntentProxy intentProxy;
@@ -268,11 +271,10 @@ public class ActivityProxy extends KrollProxy implements TiActivityResultHandler
 	public ActionBarProxy getActionBar()
 	// clang-format on
 	{
-		AppCompatActivity activity = (AppCompatActivity) getWrappedActivity();
-		if (actionBarProxy == null && activity != null) {
-			actionBarProxy = new ActionBarProxy(activity);
+		if (TiApplication.isUIThread()) {
+			return handleGetActionBar();
 		}
-		return actionBarProxy;
+		return (ActionBarProxy) TiMessenger.sendBlockingMainMessage(getMainHandler().obtainMessage(MSG_GET_ACTIONBAR));
 	}
 
 	@Kroll.method
@@ -320,6 +322,15 @@ public class ActivityProxy extends KrollProxy implements TiActivityResultHandler
 		}
 	}
 
+	private ActionBarProxy handleGetActionBar()
+	{
+		AppCompatActivity activity = (AppCompatActivity) getWrappedActivity();
+		if (actionBarProxy == null && activity != null) {
+			actionBarProxy = new ActionBarProxy(activity);
+		}
+		return actionBarProxy;
+	}
+
 	public void onResult(Activity activity, int requestCode, int resultCode, Intent data)
 	{
 		IntentProxy intent = null;
@@ -374,6 +385,11 @@ public class ActivityProxy extends KrollProxy implements TiActivityResultHandler
 				handleOpenOptionsMenu();
 				return true;
 			}
+			case MSG_GET_ACTIONBAR: {
+				AsyncResult result = (AsyncResult) msg.obj;
+				result.setResult(handleGetActionBar());
+				return true;
+			}
 		}
 		return super.handleMessage(msg);
 	}
-- 
2.7.4

